// *****************************************************************************
// Archivo: Main.cpp
// Descripcin: Sistema de control de asistencia de personal
// Notas: Ejercicio integrador de C++
// Temas includos: clases, memoria dinmica, herencia, referencias, plantillas,
//					archivos, streams, manipuladores de streams, modularizacin
//					de cdigo con metodologa de proyecto, sobrecarga de
//					funciones, sobrecarga de operadores, constructores,
//					constructores de copia, destructores, encapsulacin,
//					escritura de cdigo portable, listas, manejo de fecha/hora
// Autor: Ezequiel Gastn Miravalles
// ltima modificacin: 22/05/2009
// Website: http://www.neoegm.com/
// *****************************************************************************

/*******************************************************************************
	Copyright (C) 2009 Ezequiel Gastn Miravalles

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

/*******************************************************************************
Consigna:
---------

Hacer un sistema de control de asistencia de personal (para posterior
verificacin de ausencias y llegadas tarde con sus correspondientes
justificaciones) y su parte de gestin privada (alta de usuarios y visualizacin
de datos obtenidos a partir de la carga).

La pantalla principal debe quedar a la espera de DNI y contrasea.

Debe haber un usuario prefijado como administrador con DNI = -1234 y
Contrasea = prueba.

En el caso de ser un usuario administrador, se podr acceder a la parte de
gestin, con las siguientes opciones:

	1- Alta de usuario
	2- Lista de usuarios (datos adquiridos)
	3- Salida a archivo
	4- Cerrar la aplicacin
	0- Volver a la pantalla principal

De lo contrario, se solicitar qu tipo de accin se est realizando (Ingreso,
salida, salida a almuerzo o retorno de almuerzo) y su justificacin en el caso
de que corresponda, y se proceder a guardar dichos datos junto a la hora actual
en un registro de "Eventos" del usuario que corresponda.

Los datos necesarios de cada usuario son:
	DNI
	Nombre completo
	Contrasea
	Si es administrador
	Eventos relacionados

Se desea utilizar como base una clase "PersonaBasica" que ya fue desarrollada
por el departamento de sistemas, cuyo encabezado "PersonaBasica.h" se adjunta a
continuacin:

	#include "String.h"
	#include <iostream>

	class PersonaBasica
	{
		String m_sNombreCompleto;
		long m_nDNI;

	public:
		//Construccin
		PersonaBasica() { m_nDNI = 0; }

		//Encapsulaciones
		long DNI() const { return m_nDNI; }
		void DNI(long val) { m_nDNI = val; }

		String NombreCompleto() const { return m_sNombreCompleto; }
		void NombreCompleto(String val) { m_sNombreCompleto = val; }
	};

	std::ostream& operator<<(std::ostream& oStream, PersonaBasica& rpPersonaBasica);

Nota: no se conocen en absoluto los tamaos de los datos a ingresar y se comenta
que los usuarios suelen esribir justificaciones bastante elaboradas.
*******************************************************************************/

#include <cstdlib>
#include <iostream>

#include <iomanip>

#include "ControlDeAsistencia.h"
#include "BasicFunctions.h"
#include <fstream>

#include "cppMemDbg.h"

cppMemDbg cDbg("TestMemDbg.txt");

using namespace std;

ControlDeAsistencia ctMain;

bool PantallaPrincipal();
int PantallaAdministracion();
void PantallaAltaUsuario();
void PantallaListaUsuarios();
void PantallaAsistencia(Persona& rpPersona);
void SalidaArchivo();

int main(int argc, char *argv[])
{
	while (PantallaPrincipal())
		;
	
	//cppMemDbg tests	
	//new Persona;	//Leak
	//Persona *p = new Persona;
	//free(p);	//Bad mix
	//free(p);	//Freeing unallocated memory

	Pause();
	
	return EXIT_SUCCESS;
}

bool PantallaPrincipal()
{
	ClearScreen();

	cout << "Sistema de control de asistencia de personal" << endl;
	cout << "--------------------------------------------" << endl;
	cout << endl;
	cout << "DNI: ";

	//Leo DNI
	long nDNI;
	cin >> nDNI;
	cin.ignore();
	//-------

	Persona *pPersona = ctMain.PersonaPorDNI(nDNI);

	if (!pPersona)
	{
		cout << "DNI desconocido" << endl;

		Pause();
	}
	else
	{
		String sPassword;

		//Leo password
		cout << "Contrasea: ";
		cin >> sPassword;
		//------------

		if (pPersona->CompararPassword(sPassword))
		{
			cout << "Usuario \"" << pPersona->NombreCompleto() << "\" reconocido." << endl;

			Pause();

			if (pPersona->Admin())
			{
				//cout << "Ingresando a la interfase de administracin..." << endl;
				
				int nRet;
				
				while ((nRet = PantallaAdministracion()) == 1)
					;

				if (nRet == -1)
					return false;
				else
					return true;
			}
			else
				PantallaAsistencia(*pPersona);
		}
		else
		{
			cout << "Contrasea incorrecta." << endl;
			Pause();
		}
	}

	return true;
}

int PantallaAdministracion()
{
	ClearScreen();

	cout << "Sistema de control de asistencia de personal (ADMNISTRACION)" << endl;
	cout << "------------------------------------------------------------" << endl;
	cout << endl;
	cout << "1- Alta de usuario" << endl;
	cout << "2- Lista de usuarios (datos adquiridos)" << endl;
	cout << "3- Salida a archivo" << endl;
	cout << "4- Cerrar la aplicacin" << endl;
	cout << "0- Volver a la pantalla principal" << endl;
	cout << endl;
	cout << "Ingrese la opcin deseada: ";

	//Leo opcin
	int nOption;
	cin >> nOption;
	cin.ignore();
	//----------

	switch (nOption)
	{
	case 1:	//Alta
		PantallaAltaUsuario();
		break;
	case 2:	//Lista
		PantallaListaUsuarios();
		break;
	case 3:	//Archivo
		cout << endl;
		SalidaArchivo();
		break;
	case 4:	//Quit
		cout << "Saliendo de la aplicacin..." << endl;
		return -1;
	case 0:	//Volver
		return 0;
	default:
		cout << "Opcin no vlida." << endl;
		Pause();
		break;
	}

	return 1;
}

void PantallaAltaUsuario()
{
	ClearScreen();

	cout << "Sistema de control de asistencia de personal (Alta de usuario)" << endl;
	cout << "--------------------------------------------------------------" << endl;
	cout << endl;

	Persona pPersona;

	//Cargo DNI
	long nDNI;
	cout << "DNI: ";
	cin >> nDNI;
	cin.ignore();

	pPersona.DNI(nDNI);
	//----------

	//Variable auxiliar
	String sTmp;

	//Cargo nombre completo
	cout << "Nombre completo: ";
	cin >> sTmp;
	pPersona.NombreCompleto(sTmp);
	//---------------------

	//Cargo password
	cout << "Password: ";
	cin >> sTmp;
	pPersona.Password(sTmp);
	//--------------

	//Cargo si es administrador
	bool bAdmin;
	cout << "Administrador [1 = S/0 = No]: ";
	cin >> bAdmin;
	cin.ignore();
	pPersona.Admin(bAdmin);
	//-------------------------

	ctMain.AgregarPersona(pPersona);

	cout << endl;
	cout << "Persona agregada satisfatoriamente." << endl;

	Pause();
}

void PantallaListaUsuarios()
{
	ClearScreen();

	cout << "Sistema de control de asistencia de personal (Lista de usuarios)" << endl;
	cout << "----------------------------------------------------------------" << endl;
	cout << endl;

	for (int i = 0; i < ctMain.CantidadDePersonas(); i++)
	{
		Persona *pPersona = ctMain.PersonaItem(i);

		if (pPersona)	//Por las dudas
		{
			cout << *pPersona;
			cout << endl;
			cout << endl;
		}
	}

	//Tambin se podra haber hecho cout << ctMain.ListaPersonas(), pero quedara con un solo salto de lnea entre cada persona

	Pause();
}

void PantallaAsistencia(Persona& rpPersona)
{
	ClearScreen();

	//Imprimir encabezado con subrayado
	String sStr = String("Asistencia de \"") + rpPersona.NombreCompleto() + "\"";
	cout << sStr << endl;
	char nOld = cout.fill('-');						//Cambio caracter de relleno
	cout << setw(sStr.Largo()) << "" << endl;		//Pongo el ancho y hago output de ""
	cout.fill(nOld);								//Dejo todo como estaba
	//---------------------------------

	cout << endl;

	Asistencia aAsistencia;

	Hora hHora;
	hHora.CargarHoraActual();

	aAsistencia.HoraEvento(hHora);

	cout << "Tipo de asistencia:" << endl;
	cout << "\t1- Ingreso" << endl;
	cout << "\t2- Salida" << endl;
	cout << "\t3- Salida a almorzar" << endl;
	cout << "\t4- Vuelta de almorzar" << endl;
	
	int nSel;

	//Solicito seleccin hasta que sea vlida
	do
	{
		cout << "Seleccin: ";

		cin >> nSel;
		cin.ignore();
	} while (nSel < 1 || nSel > 4);
	//---------------------------------------
	
	aAsistencia.Tipo(nSel);

	//Variable auxiliar
	String sTmp;
	
	//Leo Justificacin
	cout << endl;
	cout << "Justificacin (si corresponde): ";
	cin >> sTmp;
	//-----------------

	aAsistencia.Justificacion(sTmp);

	cout << endl;

	if (rpPersona.AgregarEvento(aAsistencia))
		cout << "Evento agregado satisfactoriamente." << endl;
	else
		cout << "Error al intentar agregar evento." << endl;

	Pause();
}

void SalidaArchivo()
{
	cout << "Ingrese la ruta del archivo a guardar: ";

	//Leo path
	String sPath;
	cin >> sPath;
	//--------

	//Creo objeto de la clase ofstream (Output File Stream)
	ofstream fFile;
	
	//Abro archivo ubicado en el path obtenido (se abre con write access por ser ofstream)
	fFile.open(sPath);

	if (fFile.fail())	//Si no se pudo abrir
		cout << "Error al intentar abrir el archivo especificado" << endl;
	else
	{
		fFile << ctMain.ListaPersonas();
		cout << "Archivo guardado satisfactoriamente" << endl;
	}

	Pause();
}
